#!/usr/bin/env python

import codecs
import sys
import getopt

from datetime import date

try:
    import sqlite3 as db
except ImportError:
    from pysqlite2 import dbapi2 as db

SEP = "\t"

USAGE = \
'''usage:
%s <options> <relations>
where options:
-h | --help                  This text.
-a | --separator=<separator> Field separator. default: TAB
-d | --database=<database>   with <database> the SQLite 3 file
'''

def usage(msg = None, exit_code = 0):
    if msg: print >> sys.stderr, msg
    print >> sys.stderr, USAGE % sys.argv[0]
    sys.exit(exit_code)

def main():

    sep       = SEP
    database  = None
    try:
        opts, args = getopt.getopt(
            sys.argv[1:],
            "s:d:h", [ "separator=", "database=", "help" ])
    except getopt.GetoptError, err:
        usage(str(err), 1)
            
    for o, a in opts:
        if o in ('h', '-h', '--help'):
            usage()
        elif o in ('s', '-s', '--separator'):
            sep = a
        elif o in ('d', '-d', '--database'):
            database = a

    if not database:
        usage("error: missing database", 1)

    rep = sep == ' ' and '\t' or ' '

    encoder = {
        type(None): lambda s: '',
        unicode: lambda s: \
            s.replace(sep,    rep) \
             .replace('\r\n', rep) \
             .replace('\r',   rep) \
             .replace('\n',   rep),
        date: lambda s: \
            u"%d-%02d-%02d" % (s.year, s.month, s.day)
    }

    con, cur = None, None

    try:
        con = db.connect(database)
        cur = con.cursor()
        for relation in args:
            out = None
            try:
                out = codecs.open("%s.txt" % relation, "w", "UTF-8")
                stmnt = "SELECT * FROM %s" % relation
                headers = None
                cur.execute(stmnt)
                while True:
                    row = cur.fetchone()
                    if not row: break
                    if not headers:
                        headers = [desc[0] for desc in cur.description]
                        print >> out, sep.join(headers)

                    line = [encoder.get(x.__class__, lambda s: str(s))(x) \
                        for x in row]
                    print >> out, sep.join(line)
            finally:
                if out: out.close()
    finally:
        if cur: cur.close()
        if con: con.close()

if __name__ == '__main__':
    main()
