-- Das Anonymisieren von Fallakten ist nun nur noch fuer Admins moeglich
SET ROLE :db_adm;
BEGIN;
-- Log the update
INSERT INTO db_history (beschreibung, zeitpunkt) 
	VALUES ('update_1_0_2/04_anonymize.sql', current_timestamp);

-- Wrapper Funktion die alle noetigen Funktionen zum Anonymisieren einer
-- Fallakte aufruft.
CREATE OR REPLACE FUNCTION anonymize_case_wrapper(mid integer)
RETURNS void AS
$$
BEGIN
	PERFORM anonymize_case(mid);
	PERFORM delete_all_fall_dokumente(mid);
	PERFORM delete_all_fall_termine(mid);
	PERFORM delete_all_logbuch(mid);
END;
$$ LANGUAGE plpgsql VOLATILE SECURITY DEFINER;

REVOKE ALL     ON FUNCTION anonymize_case_wrapper(integer) FROM PUBLIC;
GRANT  EXECUTE ON FUNCTION anonymize_case_wrapper(integer) TO GROUP :admin_ka_group;

--
-- Funktion, die alle Anhaenge zu einem gegebenen Fall loescht
--
CREATE OR REPLACE FUNCTION delete_all_fall_dokumente(mid integer) 
RETURNS void 
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
AS $$
BEGIN
    DELETE FROM ka_dokumente_tbl WHERE master_id = mid;
END;
$$;

REVOKE ALL     ON FUNCTION delete_all_fall_dokumente(integer) FROM PUBLIC;
REVOKE ALL     ON FUNCTION delete_all_fall_dokumente(integer) FROM GROUP :cm_ka_group;
GRANT  EXECUTE ON FUNCTION delete_all_fall_dokumente(integer) TO GROUP :admin_ka_group;

--
-- Funktion, die alle Termine zu einem gegebenen Fall loescht
--
CREATE OR REPLACE FUNCTION delete_all_fall_termine(mid integer) 
RETURNS void 
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
AS $$
BEGIN
    DELETE FROM ka_termine_tbl WHERE master_id = mid;
END;
$$;

REVOKE ALL     ON FUNCTION delete_all_fall_termine(integer) FROM PUBLIC;
REVOKE ALL     ON FUNCTION delete_all_fall_termine(integer) FROM GROUP :cm_ka_group;
GRANT  EXECUTE ON FUNCTION delete_all_fall_termine(integer) TO GROUP :admin_ka_group;

--
-- Function to anonymize a case and all of its repeat groups
--
CREATE OR REPLACE FUNCTION anonymize_case(mid int)
RETURNS void AS
$$
BEGIN
    -- anonymize master table
    UPDATE master_tbl_view SET
        fkz = NULL,
        name_kompetenzagentur = NULL,
        vorname = NULL,
        name = hash_value(name, vorname),
        kundennummer = NULL,
        kontaktdaten_arge = NULL,
        kundennummer_arge = NULL,
        kontaktdaten_agentur_fuer_arbeit = NULL,
        kundennummer_agentur_fuer_arbeit = NULL,
        kundennummer_jugendamt = NULL,
        kontaktdaten_jugendamt = NULL,
        kontaktdaten_schule = NULL,
        schulname = NULL,
        kontaktdaten_gerichtshilfe = NULL,
        kundennummer_gerichtshilfe = NULL,
        kontaktdaten_weitere_institutionen = NULL,
        vorname_1 = NULL,
        name_1 = NULL,
        geburtsdatum = get_age(geburtsdatum),
        strasse = NULL,
        plz = shorten_value(plz, 3),
        wohnort = NULL,
        vorwahl = NULL,
        telefon_1 = NULL,
        mobiltelefon = NULL,
        email = NULL,
        telefon_sonstige = NULL,
        zweiter_wohnort = NULL,
        geburtsland_sonstige_nennung = NULL,
        muttersprache_sonstige_nennung = NULL,
        staatsangehoerigkeit_sonstige_nennung = NULL,
        vorname_2 = NULL,
        name_2 = NULL,
        vorwahl_1 = NULL,
        telefon_1_1 = NULL,
        mobiltelefon_1 = NULL,
        email_1 = NULL,
        physische_beeintraechtigungen = check_bool(physische_beeintraechtigungen, arzt_gutachten_physisch),
        physische_beeintraechtigungen_text = check_bool(physische_beeintraechtigungen_text, arzt_gutachten_physisch),
        psychische_beeintraechtigungen = check_bool(psychische_beeintraechtigungen, arzt_gutachten_psychisch),
        psychische_beeintraechtigungen_text = check_bool(psychische_beeintraechtigungen_text, arzt_gutachten_psychisch),
        schulabschlusserwerbsland_sonstige_nennung = NULL,
        abgebrochene_ausbildungen_text = NULL,
        weitere_ausbildungen_text_1 = NULL,
        abgebrochene_ausbildungen_text_1 = NULL,
        letzte_massnahme = NULL,
        letzte_taetigkeit = NULL,
        freie_dokumentation = NULL,
        sonstiger_foerderbedarf_nennung = NULL,
        dokumentation_zum_foerderplan = NULL,
        gruende_sonstiges_nennung_4 = NULL,
        klassenstufe_1 = NULL,
        freie_dokumentation_abschlussbewertung = NULL
    WHERE id = mid;

    -- anonymize repeat groups
    UPDATE rg_kompetenzfestellung_tbl_view SET
        dokumentation_kompetenzfeststellung = NULL
    WHERE master_id = mid;

    UPDATE rg_angebote_bildenden_bereich_tbl_view SET
        gruende_sonstiges_nennung = NULL,
        sonstiges_nachweis_bescheinigung = NULL
    WHERE master_id = mid;

    UPDATE rg_angebote_berufsvorbereitung_tbl_view SET
        gruende_sonstiges_nennung_1 = NULL,
        sonstiges_nachweis_bescheinigung_1 = NULL
    WHERE master_id = mid;

    UPDATE rg_angebote_berufliche_qualifizierung_tbl_view SET
        gruende_sonstiges_nennung_2 = NULL,
        sonstiges_nachweis_bescheinigung_2 = NULL
    WHERE master_id = mid;

    UPDATE rg_angebote_lebensbewaeltigung_tbl_view SET
        gruende_sonstiges_nennung_3 = NULL,
        sonstiges_nachweis_bescheinigung_3 = NULL
    WHERE master_id = mid;

END;
$$ LANGUAGE plpgsql VOLATILE SECURITY DEFINER;

REVOKE ALL ON FUNCTION anonymize_case(int) FROM public;
REVOKE ALL ON FUNCTION anonymize_case(int) FROM GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION anonymize_case(int) TO GROUP :admin_ka_group;
END;
