SET ROLE :db_adm;
BEGIN;
-- Log the update
INSERT INTO db_history (beschreibung, zeitpunkt) 
	VALUES ('update_1_0_2/05_documents_uuid.sql', current_timestamp);

--
-- Fuellt in der Tabelle 't_name' die Spalte 'uuid_id'
-- mit eindeutigen UUIDs.
--
CREATE OR REPLACE FUNCTION fill_uuids(t_name TEXT) 
RETURNS VOID AS 
$$
DECLARE
    t_row RECORD;
BEGIN
    FOR t_row IN EXECUTE 'SELECT id FROM ' || t_name || ' WHERE master_id is not NULL'
    LOOP
        PERFORM set_uuid(t_name, t_row.id);
    END LOOP;
END;
$$ LANGUAGE plpgsql;

-- Hinzufiegen von uuids an die bestehenden tabellen

ALTER TABLE ka_dokumente_tbl ADD COLUMN uuid_id CHAR(36);
ALTER TABLE ka_dokumente_tbl 
ADD CONSTRAINT ka_dokumente_tbl_uuid_id_key UNIQUE (uuid_id);

--
-- Tabellen mit UUIDs versehen
-- 
SELECT fill_uuids('ka_dokumente_tbl');
DROP function fill_uuids(TEXT);


-- Aendern der Funktionen die fuer das anlegen der datensaetze verantwortlich
-- sind

--
-- Funktionen zum Erstellen von Dokumenten
--

DROP FUNCTION create_fall_dokument_ds(mid integer);
CREATE OR REPLACE FUNCTION create_fall_dokument_ds(mid integer, nuuid char(36)) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE 
	rid integer;
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO ka_dokumente_tbl(master_id) VALUES (mid);
    SELECT INTO rid currval('ka_dokumente_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE ka_dokumente_tbl 
        SET    uuid_id = nuuid 
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('ka_dokumente_tbl', rid);
    END IF;
    RETURN rid;
END;
$$ ;

REVOKE ALL ON FUNCTION create_fall_dokument_ds(integer, char(36)) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_fall_dokument_ds( integer, char(36)) 
                TO GROUP :cm_ka_group;

-- Anpassen der view
DROP VIEW ka_fall_dokumente_tbl_view CASCADE;

CREATE OR REPLACE VIEW ka_fall_dokumente_tbl_view AS 
SELECT ka_dokumente_tbl.id, ka_dokumente_tbl.uuid_id, ka_dokumente_tbl.master_id,
ka_dokumente_tbl.groesse, ka_dokumente_tbl.mime, ka_dokumente_tbl.daten,
ka_dokumente_tbl.name FROM ka_dokumente_tbl 
INNER JOIN master_tbl ON
ka_dokumente_tbl.master_id = master_tbl.id 
WHERE master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
	    SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
	    WHERE nm2.benutzer_id = getuserid()
	)
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE ka_fall_dokumente_delete_rule AS ON DELETE TO 
ka_fall_dokumente_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_fall_dokumente_insert_rule AS ON INSERT TO 
ka_fall_dokumente_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_fall_dokumente_update_rule AS ON UPDATE TO ka_fall_dokumente_tbl_view 
	DO INSTEAD  UPDATE ka_dokumente_tbl SET 
	groesse = new.groesse, 
	mime = new.mime, 
	daten = new.daten, 
	name = new.name 
	WHERE ka_dokumente_tbl.id = new.id;

GRANT SELECT ON ka_fall_dokumente_tbl_view TO GROUP :cm_ka_group;
GRANT UPDATE ON ka_fall_dokumente_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_fall_dokumente_tbl_view TO GROUP :admin_ka_group;

COMMIT;
