BEGIN;

-- Das wird als postgres ausgefuehrt, weil :admin_ka_group
-- aus noch ungeklaerten Gruenden keine Python-Funktionen
-- anlegen kann.

--
-- Generiert eine UUID
--
CREATE OR REPLACE FUNCTION generate_uuidgen()
RETURNS  TEXT AS
$BODY$
    from commands import getstatusoutput
    from random   import randint
    try:
        result = getstatusoutput("/usr/bin/uuid -v4").strip()
        if result[0] != 0: raise Exception()
        return result[1]
    except:
        return "%08x-%04x-%04x-%04x-%012x" % (
            randint(0, 1 <<  8*4),
            randint(0, 1 <<  4*4),
            randint(0, 1 <<  4*4),
            randint(0, 1 <<  4*4),
            randint(0, 1 << 12*4))
$BODY$
LANGUAGE 'plpythonu' VOLATILE;

GRANT EXECUTE ON FUNCTION generate_uuidgen() TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION generate_uuidgen() TO GROUP :admin_ka_group;
ALTER FUNCTION generate_uuidgen() OWNER TO :adm_ka_owner;

-- Ab hier als :adm_ka_owner

SET ROLE :adm_ka_owner;

--
-- Setzt in der Tabelle 't_name' in der Spalte mit der
-- ID 'id' das Feld 'uuid_id' mit einer UUID.
-- Bei Verletzung des Unique-Constraints wird bis
-- zu 500 mal neu gewuerfelt.
--
CREATE OR REPLACE FUNCTION set_uuid(t_name TEXT, id int) 
RETURNS VOID AS 
$$
DECLARE
    xuuid_id CHAR(36);
BEGIN
    FOR i IN 0..500 LOOP
        SELECT INTO xuuid_id generate_uuidgen();
        BEGIN
            EXECUTE 'UPDATE '           || t_name   || 
                    ' SET uuid_id = ''' || xuuid_id ||
                    ''' WHERE id = '    || id;
        EXCEPTION WHEN unique_violation THEN
            CONTINUE;
        END;
        RETURN;
    END LOOP;
    RAISE EXCEPTION 'set_uuid failed';
END;
$$ LANGUAGE plpgsql;
REVOKE ALL ON FUNCTION set_uuid(TEXT, INT) FROM public;


--
-- Fuellt in der Tabelle 't_name' die Spalte 'uuid_id'
-- mit eindeutigen UUIDs.
--
CREATE OR REPLACE FUNCTION fill_uuids(t_name TEXT) 
RETURNS VOID AS 
$$
DECLARE
    t_row RECORD;
BEGIN
    FOR t_row IN EXECUTE 'SELECT id FROM ' || t_name
    LOOP
        PERFORM set_uuid(t_name, t_row.id);
    END LOOP;
END;
$$ LANGUAGE plpgsql;

--
-- Spalten hinzufuegen
-- 
ALTER TABLE master_tbl
    ADD COLUMN uuid_id CHAR(36);

ALTER TABLE rg_kompetenzfestellung_tbl
    ADD COLUMN uuid_id CHAR(36);

ALTER TABLE rg_angebote_bildenden_bereich_tbl
    ADD COLUMN uuid_id CHAR(36);

ALTER TABLE rg_angebote_berufsvorbereitung_tbl
    ADD COLUMN uuid_id CHAR(36);

ALTER TABLE rg_angebote_berufliche_qualifizierung_tbl 
    ADD COLUMN uuid_id CHAR(36);

ALTER TABLE rg_angebote_lebensbewaeltigung_tbl
    ADD COLUMN uuid_id CHAR(36);

--
-- Unique Constraints ergaenzen
--
ALTER TABLE master_tbl
    ADD CONSTRAINT master_tbl_uuid_id_key UNIQUE (uuid_id);

ALTER TABLE rg_kompetenzfestellung_tbl
    ADD CONSTRAINT rg_kompetenzfestellung_tbl_uuid_id_key UNIQUE (uuid_id);

ALTER TABLE rg_angebote_bildenden_bereich_tbl
    ADD CONSTRAINT rg_angebote_bildenden_bereich_tbl_uuid_id_key UNIQUE (uuid_id);

ALTER TABLE rg_angebote_berufsvorbereitung_tbl
    ADD CONSTRAINT rg_angebote_berufsvorbereitung_tbl_uuid_id_key UNIQUE (uuid_id);

ALTER TABLE rg_angebote_berufliche_qualifizierung_tbl
    ADD CONSTRAINT rg_angebote_berufliche_qualifizierung_tbl_uuid_id_key UNIQUE (uuid_id);

ALTER TABLE rg_angebote_lebensbewaeltigung_tbl
    ADD CONSTRAINT rg_angebote_lebensbewaeltigung_tbl_uuid_id_key UNIQUE (uuid_id);

--
-- Tabellen mit UUIDs versehen
-- 
SELECT fill_uuids('master_tbl');
SELECT fill_uuids('rg_kompetenzfestellung_tbl');
SELECT fill_uuids('rg_angebote_bildenden_bereich_tbl');
SELECT fill_uuids('rg_angebote_berufsvorbereitung_tbl');
SELECT fill_uuids('rg_angebote_berufliche_qualifizierung_tbl');
SELECT fill_uuids('rg_angebote_lebensbewaeltigung_tbl');

--
-- Die Fuell-Funktion wird nach dem Update nicht mehr gebraucht.
--
DROP FUNCTION fill_uuids(TEXT);

--
-- create Funktionen um UUID parameter erweitern.
--

-- Die parameterlose Variante loeschen.

DROP FUNCTION create_master_ds();

-- Funktionen zum Anlegen neuer (Teile von) Fallakten
-- Diese sind nur fuer die Gruppe CM_KA aufrufbar (Admin_KA darf keine neuen 
-- Akten anlegen) und laufen mit den Berechtigungen des zentralen
-- administrativen Benutzers der KA
CREATE OR REPLACE FUNCTION create_master_ds(nuuid CHAR(36)) 
RETURNS INTEGER
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS 
$$
DECLARE
    standin INTEGER;
    mid     INTEGER;
BEGIN
    INSERT INTO master_tbl (bearbeiter_id) VALUES (getuserid());
    SELECT INTO standin vertreter from ka_benutzer_tbl WHERE id = getuserid();

    IF standin IS NOT NULL THEN 
        INSERT INTO nm_benutzer_master_tbl(benutzer_id, master_id) VALUES (standin, 
    currval('master_tbl_id_seq'));
    END IF;

    SELECT INTO mid currval('master_tbl_id_seq'); 

    IF nuuid IS NOT NULL THEN
        UPDATE master_tbl SET uuid_id = nuuid WHERE id = mid;
    ELSE
        PERFORM set_uuid('master_tbl', mid);
    END IF;

    PERFORM create_status_ds(mid);
    RETURN mid;
END;
$$;

GRANT EXECUTE ON FUNCTION create_master_ds(CHAR(36))
    TO GROUP :cm_ka_group;

-- Repeat groups

--
-- Unterstuetzungsangebote im Bereich der Berufsvorbereitung
--

DROP FUNCTION create_rg_beruf_ds(INTEGER);

CREATE OR REPLACE FUNCTION create_rg_beruf_ds(mid INTEGER, nuuid CHAR(36)) 
RETURNS INTEGER
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS 
$$
DECLARE
    rid INTEGER;
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_angebote_berufsvorbereitung_tbl(master_id) VALUES (mid);

    SELECT INTO rid currval('rg_angebote_berufsvorbereitung_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_angebote_berufsvorbereitung_tbl 
        SET    uuid_id = nuuid 
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_angebote_berufsvorbereitung_tbl', rid);
    END IF;

    RETURN rid;
END;
$$ ;

GRANT EXECUTE ON FUNCTION create_rg_beruf_ds(INTEGER, CHAR(36)) 
    TO GROUP :cm_ka_group;

--
-- Unterstuetzungsangebote im allgemein bildenden Bereich
--
DROP FUNCTION create_rg_bilden_ds(INTEGER);

CREATE OR REPLACE FUNCTION create_rg_bilden_ds(mid INTEGER, nuuid CHAR(36)) 
RETURNS INTEGER
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS 
$$
DECLARE
    rid INTEGER;
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_angebote_bildenden_bereich_tbl(master_id) VALUES (mid);

    SELECT INTO rid currval('rg_angebote_bildenden_bereich_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_angebote_bildenden_bereich_tbl 
        SET    uuid_id = nuuid 
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_angebote_bildenden_bereich_tbl', rid);
    END IF;

    RETURN rid;
END;
$$ ;

GRANT EXECUTE ON FUNCTION create_rg_bilden_ds(INTEGER, CHAR(36))
    TO GROUP :cm_ka_group;


--
-- Unterstuetzungsangebote im Bereich der Lebensbewaeltigung
--
DROP FUNCTION create_rg_leben_ds(INTEGER);

CREATE OR REPLACE FUNCTION create_rg_leben_ds(mid INTEGER, nuuid CHAR(36)) 
RETURNS INTEGER
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid INTEGER;
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    INSERT INTO rg_angebote_lebensbewaeltigung_tbl(master_id) VALUES (mid);

    SELECT INTO rid currval('rg_angebote_lebensbewaeltigung_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_angebote_lebensbewaeltigung_tbl 
        SET    uuid_id = nuuid 
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_angebote_lebensbewaeltigung_tbl', rid);
    END IF;

    RETURN rid;
END;
$$ ;

GRANT EXECUTE ON FUNCTION create_rg_leben_ds(INTEGER, CHAR(36))
    TO GROUP :cm_ka_group;

--
-- Unterstuetzungsangebote im Bereich der beruflichen Qualifizierung
--
DROP FUNCTION create_rg_quali_ds(INTEGER);

CREATE OR REPLACE FUNCTION create_rg_quali_ds(mid INTEGER, nuuid CHAR(36)) 
RETURNS INTEGER
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid INTEGER;
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    INSERT INTO rg_angebote_berufliche_qualifizierung_tbl (master_id) VALUES (mid);

    SELECT INTO rid currval('rg_angebote_berufliche_qualifizierung_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_angebote_berufliche_qualifizierung_tbl 
        SET    uuid_id = nuuid 
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_angebote_berufliche_qualifizierung_tbl', rid);
    END IF;

    RETURN rid;
END;
$$ ;

GRANT EXECUTE ON FUNCTION create_rg_quali_ds(INTEGER, CHAR(36))
    TO GROUP :cm_ka_group;

--
-- Kompetenzfeststellung
--
DROP FUNCTION create_rg_kompetenz_ds(INTEGER);

CREATE OR REPLACE FUNCTION create_rg_kompetenz_ds(mid INTEGER, nuuid CHAR(36)) 
RETURNS INTEGER
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS 
$$
DECLARE
    rid INTEGER;
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_kompetenzfestellung_tbl(master_id) VALUES (mid);

    SELECT INTO rid currval('rg_kompetenzfestellung_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_kompetenzfestellung_tbl 
        SET    uuid_id = nuuid 
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_kompetenzfestellung_tbl', rid);
    END IF;

    RETURN rid;
END;
$$ ;

GRANT EXECUTE ON FUNCTION create_rg_kompetenz_ds(INTEGER, CHAR(36))
    TO GROUP :cm_ka_group;

COMMIT;
