--
-- Die Funktion sanitize_max_storage_time() setzt die maximale
-- Aufbewahrungsdauer auf 90 Tage, wenn dieser Wert nicht existieren
-- sollte. Ist der Wert kein Integer, wird er auf '90' gesetzt.
-- Ist der Wert kleiner 7, wird er auch auf '7' gesetzt.
-- Ist der Wert groesser 180, wird er auf '180' gesetzt.
-- Andere Werte werden so belassen, wie sie vorgefunden wurden.
--
BEGIN;
SET ROLE :db_adm;

CREATE OR REPLACE FUNCTION sanitize_max_storage_time()
RETURNS void AS
$$
DECLARE
    fix_it       BOOLEAN;
    max_age_text text;
    max_age      int;
    fix_value    text;
BEGIN
    SELECT INTO max_age_text wert FROM ka_konfiguration_tbl_view WHERE bez = 'max_speicherdauer';
    IF NOT FOUND THEN
        INSERT INTO ka_konfiguration_tbl (bez, wert) VALUES ('max_speicherdauer', '90');
        RETURN;
    END IF;

    fix_it    := FALSE;
    fix_value := '90';
    BEGIN
        max_age := max_age_text::int;
        IF max_age < 7 THEN
            fix_it    := TRUE;
            fix_value := '7';
        ELSIF max_age > 180 THEN
            fix_it    := TRUE;
            fix_value := '180';
        END IF;
    EXCEPTION WHEN OTHERS THEN
        fix_it := TRUE;
    END;

    IF fix_it THEN
        UPDATE ka_konfiguration_tbl_view SET wert = fix_value WHERE bez = 'max_speicherdauer';
    END IF;
END;
$$ LANGUAGE plpgsql;

SELECT sanitize_max_storage_time();

DROP FUNCTION sanitize_max_storage_time();

COMMIT;
