<?xml version="1.0" encoding="UTF-8"?>
<!-- 
   replace-enums.xsl

   Autor: Sascha L. Teichmann <teichmann@intevation.de>
   (c) 2008 Intevation GmbH

   Einige Felder in den von WASKA erzeugten XML-Akten repraesentieren
   ihre Werte als Ganzzahlwerte, die im WASKA-XML-Schema in dafuer 
   vorgesehenen Tabellen auf zugeordnete Texte abgebildet werden koennen.

   Diese Transformation ersetzt in XML-Fallakten die Indexwerte
   durch die zugeordneten textuellen Beschreibungen.

   Eingabe: Ein zum WASKA-Schema konformes XML-Dokument.
   Ausgabe: Ein Dokument, in dem Aufzaehlungswerte durch
            den ihnen zugeordnete Klartexte ersetzt wurden.

            !!! Das resultieren XML-Dokument ist nicht mehr
            !!! konform zum WASKA-Schema und kann folglich
            !!! auch nicht wieder in WASKA importiert werden.

   Als Standard-Schema wird Das WASKA-Schema 1.4 angenommen,
   das von http://waska-anwender.intevation.de/schemas/waska-1.4.xsd
   heruntergeladen wird. Ueber den externen Parameter 'schema-url' 
   kann eine andere URL angegeben werden. Zur Uebergabe dieses 
   Parameters lesen Sie bitte die Dokumentation Ihres XSLT-Prozessors.
  -->
<xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xhtml="http://www.w3.org/1999/xhtml">

    <xsl:output encoding="UTF-8" method="xml"/>

    <!-- WASKA XML-Schema -->
    <xsl:param name="schema-url">http://waska-anwender.intevation.de/schemas/waska-1.4.xsd</xsl:param>

    <!-- Index zum Zugriff auf die Ersetzungen der Felder,
         die mit Hilfe von Aufzaehlungen definiert sind -->
    <xsl:key 
        name="enum-entries"
        match="xs:simpleType[starts-with(@name, 'enum_')]/xs:annotation/xs:documentation/xhtml:table/xhtml:tr[position() > 1]/xhtml:td[2]"
        use="concat(../../../../../@name, ' ', ../xhtml:td[1]/text())" />

    <!-- Index zur Ermittlung der Basistypen. Wird benutzt, um herauszufinden, 
         ob ein Feld durch einen Aufzaehlungstyp definiert ist. -->
    <xsl:key
        name="base-types"
        match="xs:simpleType[not(starts-with(@name, 'enum_'))]/xs:restriction/@base[starts-with(., 'enum_')]"
        use="../../@name" />

    <!-- Das WASKA-Schema laden -->
    <xsl:variable name="waska-schema" select="document($schema-url)"/>

    <!-- Hilfs-Template, um im Aufzaehlungs-Index den Ersatztext zu finden -->
    <xsl:template name="lookup-enum-entry">
        <xsl:param name="enum-type"/>
        <xsl:param name="value"/>
        <xsl:for-each select="$waska-schema">
            <xsl:value-of select="key('enum-entries', concat($enum-type, ' ', $value))"/>
        </xsl:for-each>
    </xsl:template>

    <!-- Hilfs-Template, das den Basis-Typ zum uebergebenen Knotennamen ermittelt -->
    <xsl:template name="lookup-base-type">
        <xsl:param name="node-name"/>
        <xsl:for-each select="$waska-schema">
            <xsl:value-of select="key('base-types', concat($node-name, '_type'))"/>
        </xsl:for-each>
    </xsl:template>

    <!-- Identitaets-Transformation -->
    <xsl:template match="node()|@*">
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>

    <!-- Textuellen Inhalte der Knoten pruefen, ob sie zu einen
         Aufzaehlungstyp gehoeren oder nicht.
         Falls ja, wird der Inhalt durch den zugeordneten Klartext ersetzt.
         Falls nicht, wird der Inhalt durchkopiert. -->
    <xsl:template match="text()">
        <xsl:variable name="base-type">
            <xsl:call-template name="lookup-base-type">
                <xsl:with-param name="node-name" select="name(..)"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:choose>
            <xsl:when test="string-length($base-type)">
                <xsl:call-template name="lookup-enum-entry">
                    <xsl:with-param name="enum-type" select="$base-type"/>
                    <xsl:with-param name="value" select="."/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="."/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>
